@@ -29,12 +29,14 @@
 #include <sstream>
 #include <utility>
 
-#include "image-display.h"
+#include "graphics-magick-source.h"
 #include "jpeg-source.h"
 #include "openslide-source.h"
+#include "pdf-image-source.h"
 #include "qoi-image-source.h"
 #include "stb-image-source.h"
-#include "video-display.h"
+#include "svg-image-source.h"
+#include "video-source.h"
 
 namespace timg {
 
@@ -176,8 +178,22 @@ ImageSource *ImageSource::Create(const std::string &filename,
         }
 #endif
 
+#ifdef WITH_TIMG_RSVG
+        result.reset(new SVGImageSource(filename));
+        if (result->LoadAndScale(options, frame_offset, frame_count)) {
+            return result.release();
+        }
+#endif
+
+#ifdef WITH_TIMG_POPPLER
+        result.reset(new PDFImageSource(filename));
+        if (result->LoadAndScale(options, frame_offset, frame_count)) {
+            return result.release();
+        }
+#endif
+
 #ifdef WITH_TIMG_GRPAPHICSMAGICK
-        result.reset(new ImageLoader(filename));
+        result.reset(new GraphicsMagickSource(filename));
         if (result->LoadAndScale(options, frame_offset, frame_count)) {
             return result.release();
         }
@@ -194,7 +210,7 @@ ImageSource *ImageSource::Create(const std::string &filename,
 
 #ifdef WITH_TIMG_VIDEO
     if (attempt_video_loading) {
-        result.reset(new VideoLoader(filename));
+        result.reset(new VideoSource(filename));
         if (result->LoadAndScale(options, frame_offset, frame_count)) {
             return result.release();
         }
@@ -226,7 +242,8 @@ ImageSource *ImageSource::Create(const std::string &filename,
 #ifndef WITH_TIMG_VIDEO
     if (error->empty()) {
         const char *const end_filename = filename.data() + filename.length();
-        for (const char *suffix : {".mov", ".mp4", ".mkv", ".avi", ".wmv"}) {
+        for (const char *suffix :
+             {".mov", ".mp4", ".mkv", ".avi", ".wmv", ".webm"}) {
             const size_t suffix_len = strlen(suffix);
             if (filename.length() < suffix_len) continue;
             if (strcasecmp(end_filename - suffix_len, suffix) == 0) {
