@@ -18,6 +18,8 @@
 #include <string.h>
 #include <unistd.h>
 
+#include <memory>
+
 namespace timg {
 std::string Renderer::TrimTitle(const std::string &title,
                                 int requested_width) const {
@@ -39,8 +41,8 @@ namespace {
 class SingleColumnRenderer final : public Renderer {
 public:
     SingleColumnRenderer(timg::TerminalCanvas *canvas,
-                         const DisplayOptions &display_opts)
-        : Renderer(canvas, display_opts) {}
+                         const DisplayOptions &display_opts, Duration wait_time)
+        : Renderer(canvas, display_opts), wait_time_(wait_time) {}
 
     WriteFramebufferFun render_cb(const std::string &title) final {
         // For single column mode, implementation is straightforward
@@ -51,6 +53,12 @@ public:
         };
     }
 
+    void MaybeWaitBetweenImageSources() const final {
+        if (!wait_time_.is_zero()) {
+            (Time::Now() + wait_time_).WaitUntil();
+        }
+    }
+
 private:
     void RenderTitle(const std::string &title) {
         if (!options_.show_title) return;
@@ -58,6 +66,8 @@ private:
             TrimTitle(title, options_.width / options_.cell_x_px);
         canvas_->AddPrefixNextSend(tout.data(), tout.size());
     }
+
+    const Duration wait_time_;
 };
 
 // The multi column renderer positions every update in a new column.
@@ -66,10 +76,15 @@ private:
 class MultiColumnRenderer final : public Renderer {
 public:
     MultiColumnRenderer(timg::TerminalCanvas *canvas,
-                        const DisplayOptions &display_opts, int cols, int rows)
+                        const DisplayOptions &display_opts, int cols, int rows,
+                        Duration wait_between_images,
+                        Duration wait_between_rows)
+
         : Renderer(canvas, display_opts),
           columns_(cols),
-          column_width_(display_opts.width) {}
+          column_width_(display_opts.width),
+          wait_between_images_(wait_between_images),
+          wait_between_rows_(wait_between_rows) {}
 
     ~MultiColumnRenderer() final {
         if (current_column_ != 0) {
@@ -81,29 +96,27 @@ public:
     }
 
     WriteFramebufferFun render_cb(const std::string &title) final {
-        ++current_column_;
-        if (current_column_ >= columns_) {
-            // If our current image is shorter than the previous one,
-            // we need to make up the difference to be ready for the next
-            const int down = highest_fb_column_height_ - last_fb_height_;
-            if (down > 0) canvas_->MoveCursorDY(down);
-            current_column_           = 0;
-            highest_fb_column_height_ = 0;
-        }
-
+        AdvanceColumn();
         PrepareTitle(title);
         first_render_call_ = true;
         return [this](int x, int dy, const Framebuffer &fb, SeqType seq_type,
                       const Duration &end_of_frame) {
-            const int x_offset = current_column_ * column_width_;
             int y_offset;
             if (first_render_call_) {
                 // Unless we're in the first column, we've to move up from last
                 y_offset = current_column_ > 0 ? -last_fb_height_ : 0;
             }
             else {
-                y_offset = dy;
+                if (dy != 0) {
+                    y_offset = dy;  // Some animation jumping back
+                }
+                else {
+                    // Multi-image rendering, shown in multiple columns.
+                    y_offset = AdvanceColumn() ? 0 : -last_fb_height_;
+                }
             }
+
+            const int x_offset = current_column_ * column_width_;
             if (options_.show_title && first_render_call_) {
                 // If we have to show the headline, we've to do the move up
                 // before Send() would do it.
@@ -111,10 +124,10 @@ public:
                 if (y_offset) {
                     // Move pixels needed for cell, then one more
                     // then one line more to reach the place of the title.
-                    const int y_move = (-y_offset + options_.cell_y_px - 1) /
-                                       options_.cell_y_px;
+                    const int y_move =
+                        canvas_->cell_height_for_pixels(y_offset);
                     const int kSpaceForTitle = 1;
-                    canvas_->MoveCursorDY(-y_move - kSpaceForTitle);
+                    canvas_->MoveCursorDY(y_move - kSpaceForTitle);
                 }
                 canvas_->MoveCursorDX(x_offset / options_.cell_x_px);
                 canvas_->AddPrefixNextSend(title_.c_str(), title_.length());
@@ -129,14 +142,40 @@ public:
         };
     }
 
+    void MaybeWaitBetweenImageSources() const final {
+        if (!wait_between_images_.is_zero()) {
+            (Time::Now() + wait_between_images_).WaitUntil();
+        }
+        if (current_column_ == columns_ - 1 && !wait_between_rows_.is_zero()) {
+            (Time::Now() + wait_between_rows_).WaitUntil();
+        }
+    }
+
 private:
     void PrepareTitle(const std::string &title) {
         if (!options_.show_title) return;
         title_ = TrimTitle(title, column_width_ / options_.cell_x_px);
     }
 
+    bool AdvanceColumn() {
+        ++current_column_;
+        if (current_column_ >= columns_) {
+            // If our current image is shorter than the previous one,
+            // we need to make up the difference to be ready for the next
+            const int down = highest_fb_column_height_ - last_fb_height_;
+            if (down > 0) canvas_->MoveCursorDY(down);
+            current_column_           = 0;
+            highest_fb_column_height_ = 0;
+            return true;
+        }
+        return false;
+    }
+
     const int columns_;
     const int column_width_;
+    const Duration wait_between_images_;
+    const Duration wait_between_rows_;
+
     std::string title_;
     bool first_render_call_       = true;
     int current_column_           = -1;
@@ -152,12 +191,16 @@ Renderer::Renderer(timg::TerminalCanvas *canvas,
 
 std::unique_ptr<Renderer> Renderer::Create(timg::TerminalCanvas *output,
                                            const DisplayOptions &display_opts,
-                                           int cols, int rows) {
+                                           int cols, int rows,
+                                           Duration wait_between_images,
+                                           Duration wait_between_rows) {
     if (cols > 1) {
         return std::make_unique<MultiColumnRenderer>(output, display_opts, cols,
-                                                     rows);
+                                                     rows, wait_between_images,
+                                                     wait_between_rows);
     }
-    return std::make_unique<SingleColumnRenderer>(output, display_opts);
+    return std::make_unique<SingleColumnRenderer>(
+        output, display_opts, std::max(wait_between_images, wait_between_rows));
 }
 
 }  // namespace timg
