@@ -19,6 +19,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/ioctl.h>
 #include <sys/select.h>
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -86,11 +87,12 @@ static const char *QueryTerminal(const char *query, char *const buffer,
     atexit(clean_up_terminal);
 
     const int query_len = strlen(query);
-    if (write(s_tty_fd, query, query_len) != query_len)
+    if (write(s_tty_fd, query, query_len) != query_len) {
         return nullptr;  // Don't bother. We're best effort here.
+    }
 
     const char *found_pos     = nullptr;
-    size_t available          = buflen;
+    size_t available          = buflen - 1;  // Allow for nul termination.
     char *pos                 = buffer;
     timg::Time now            = Time::Now();
     const timg::Time deadline = now + time_budget;
@@ -106,6 +108,7 @@ static const char *QueryTerminal(const char *query, char *const buffer,
         pos += r;
         available -= r;
         const size_t total_read = pos - buffer;
+        *pos                    = '\0';  // nul-terminate for c-functions.
         found_pos               = response_found_p(buffer, total_read);
         if (found_pos) break;
         now = Time::Now();
@@ -188,12 +191,27 @@ TermGraphicsInfo QuerySupportedGraphicsProtocol() {
     result.known_broken_sixel_cursor_placement = false;
     result.in_tmux                             = false;
 
+    // Environment variables can be changed, so guesses from environment
+    // variables are just that: guesses.
+    // They will help as a fallback if the [>q query does not work.
+    // Only testing environment variables with very specific content.
+
     const char *const term = getenv("TERM");
-    if (term && strcmp(term, "xterm-kitty") == 0) {
+    if (term && ((strcmp(term, "xterm-kitty") == 0) ||
+                 (strcmp(term, "xterm-ghostty") == 0))) {
         result.preferred_graphics = GraphicsProtocol::kKitty;
         // Fall through, as we still have to determine if we're in tmux
     }
 
+    // vscode doesn't provide a way to query the terminal, but can do
+    // iterm graphics.
+    const char *const term_program = getenv("TERM_PROGRAM");
+    if (term_program && strcmp(term_program, "vscode") == 0) {
+        result.preferred_graphics = GraphicsProtocol::kIterm2;
+        // In case the user chooses sixel.
+        result.known_broken_sixel_cursor_placement = true;
+    }
+
     const Duration kTimeBudget = Duration::Millis(250);
 
     // We send out two queries: one CSI for terminal version detection that
@@ -223,6 +241,9 @@ TermGraphicsInfo QuerySupportedGraphicsProtocol() {
                       if (contains(data, len, "kitty")) {
                           result.preferred_graphics = GraphicsProtocol::kKitty;
                       }
+                      if (contains(data, len, "ghostty")) {
+                          result.preferred_graphics = GraphicsProtocol::kKitty;
+                      }
                       if (contains(data, len, "mlterm")) {
                           result.preferred_graphics = GraphicsProtocol::kSixel;
                       }
@@ -239,4 +260,57 @@ TermGraphicsInfo QuerySupportedGraphicsProtocol() {
     return result;
 }
 
+static bool QueryCellWidthHeight(int *width, int *height) {
+    const Duration kTimeBudget      = Duration::Millis(50);
+    constexpr char query[]          = "\033[16t";
+    constexpr char response_start[] = "\033[6;";
+    char buffer[512];
+    const char *const result = QueryTerminal(
+        query, buffer, sizeof(buffer), kTimeBudget,
+        [response_start](const char *data, size_t len) -> const char * {
+            return (const char *)memmem(data, len, response_start,
+                                        strlen(response_start));
+        });
+    int w, h;
+    if (!result ||
+        sscanf(result + strlen(response_start), "%d;%dt", &h, &w) != 2) {
+        return false;
+    }
+    *width  = w;
+    *height = h;
+    return true;
+}
+
+// Probe all file descriptors that might be connect to tty for term size.
+TermSizeResult DetermineTermSize() {
+    TermSizeResult result;
+    for (int fd : {STDOUT_FILENO, STDERR_FILENO, STDIN_FILENO}) {
+        struct winsize w = {};
+        if (ioctl(fd, TIOCGWINSZ, &w) != 0) continue;
+        // If we get the size of the terminals in pixels, we can determine
+        // what aspect ratio the pixels. This is also needed to
+        // jump up the exact number of character cells needed for animations
+        // and grid display.
+        //
+        // If TIOCGWINSZ does not return ws_xpixel/ws_ypixel, we attempt to
+        // query the terminal.
+        if (w.ws_xpixel >= 2 * w.ws_col && w.ws_ypixel >= 4 * w.ws_row &&
+            w.ws_col > 0 && w.ws_row > 0) {
+            // Infer the font size if we have window pixel size available
+            // after a plausibility check indicates that values look good.
+            result.font_width_px  = w.ws_xpixel / w.ws_col;
+            result.font_height_px = w.ws_ypixel / w.ws_row;
+        }
+        else {
+            // Alright, TIOCGWINSZ did not return the terminal size, let's
+            // see if it reports character cell size otherwise
+            QueryCellWidthHeight(&result.font_width_px, &result.font_height_px);
+        }
+        result.cols = w.ws_col;
+        result.rows = w.ws_row;
+        break;
+    }
+    return result;
+}
+
 }  // namespace timg
