@@ -24,36 +24,42 @@
 #include "iterm2-canvas.h"
 #include "kitty-canvas.h"
 #include "renderer.h"
+#include "term-query.h"
 #include "terminal-canvas.h"
-#include "termutils.h"
 #include "thread-pool.h"
 #include "timg-help.h"
 #include "timg-time.h"
 #include "timg-version.h"
 #include "unicode-block-canvas.h"
+#include "utils.h"
+
 #ifdef WITH_TIMG_SIXEL
 #    include "sixel-canvas.h"
 #endif
 
-#ifdef WITH_TIMG_TERMINAL_QUERY
-#    include "term-query.h"
-#endif
-
 // To display version number
 #ifdef WITH_TIMG_OPENSLIDE_SUPPORT
 #    include "openslide-source.h"
 #endif
 #ifdef WITH_TIMG_VIDEO
-#    include "video-display.h"
+#    include "video-source.h"
 #endif
 #ifdef WITH_TIMG_GRPAPHICSMAGICK
 #    include <Magick++.h>
 
-#    include "image-display.h"
+#    include "graphics-magick-source.h"
 #endif
 #ifdef WITH_TIMG_SIXEL
 #    include <sixel.h>
 #endif
+#ifdef WITH_TIMG_RSVG
+#    include <cairo-version.h>
+#    include <librsvg/rsvg.h>
+#endif
+#ifdef WITH_TIMG_POPPLER
+#    include <cairo-version.h>
+#    include <poppler.h>
+#endif
 
 #include <errno.h>
 #include <fcntl.h>
@@ -75,6 +81,7 @@
 #include <iostream>
 #include <memory>
 #include <mutex>
+#include <optional>
 #include <string>
 #include <thread>
 #include <vector>
@@ -221,9 +228,10 @@ static int usage(const char *progname, ExitCode exit_code, int width,
         "fit.\n"
         "\t                 Optional parameter 'i' only enlarges in integer "
         "steps.\n"
-        "\t--clear        : Clear screen first. Optional argument 'every' "
+        "\t--clear[=every]: Clear draw area first. Optional argument 'every' "
         "will\n"
-        "\t                 clear before every image (useful with -w)\n"
+        "\t                 clear before every image (useful with -w/-wr,\n"
+        "\t                 but not with --grid)\n"
 #ifdef WITH_TIMG_VIDEO
         "\t-V             : Directly use Video subsystem. Don't probe image\n"
         "\t                 decoding first (useful, if you stream video from "
@@ -255,7 +263,7 @@ static int usage(const char *progname, ExitCode exit_code, int width,
         "\t--help         : Page through detailed manpage-like help and exit.\n"
 
         "\n  \e[1mScrolling\e[0m\n"
-        "\t--scroll=[<ms>]       : Scroll horizontally (optionally: delay ms "
+        "\t--scroll[=<ms>]       : Scroll horizontally (optionally: delay ms "
         "(60)).\n"
         "\t--delta-move=<dx:dy>  : delta x and delta y when scrolling "
         "(default:1:0)\n"
@@ -305,11 +313,11 @@ bool AppendToFileList(const std::string &filelist_file,
     return true;
 }
 
-static void PresentImages(LoadedImageSources *loaded_sources,
-                          const timg::DisplayOptions &display_opts,
-                          const timg::PresentationOptions &present,
-                          timg::BufferedWriteSequencer *sequencer,
-                          bool *any_animations_seen) {
+static int PresentImages(LoadedImageSources *loaded_sources,
+                         const timg::DisplayOptions &display_opts,
+                         const timg::PresentationOptions &present,
+                         timg::BufferedWriteSequencer *sequencer,
+                         bool *any_animations_seen) {
     using timg::ThreadPool;
     std::unique_ptr<ThreadPool> compression_pool;
     std::unique_ptr<TerminalCanvas> canvas;
@@ -371,11 +379,13 @@ static void PresentImages(LoadedImageSources *loaded_sources,
     };
 
     // Showing them in order of files on the command line.
-    bool is_first = true;
+    bool is_first    = true;
+    int valid_images = 0;
     for (auto &source_future : *loaded_sources) {
         if (interrupt_received) break;
         std::unique_ptr<timg::ImageSource> source(source_future.get());
         if (!source) continue;
+        valid_images++;
         *any_animations_seen |= source->IsAnimationBeforeFrameLimit();
         before_image_show(is_first);
         source->SendFrames(present.duration_per_image, present.loops,
@@ -387,6 +397,97 @@ static void PresentImages(LoadedImageSources *loaded_sources,
         is_first = false;
     }
     sequencer->Flush();
+    return valid_images;
+}
+
+static std::optional<Pixelation> ParsePixelation(const char *as_text) {
+    if (!as_text) return std::nullopt;
+    switch (tolower(as_text[0])) {
+    case 'h': return Pixelation::kHalfBlock; break;
+    case 'q': return Pixelation::kQuarterBlock; break;
+    case 'k': return Pixelation::kKittyGraphics; break;
+    case 'i': return Pixelation::kiTerm2Graphics; break;
+#ifdef WITH_TIMG_SIXEL
+    case 's': return Pixelation::kSixelGraphics; break;
+#endif
+    default: return std::nullopt;
+    }
+}
+
+static const char *PixelationToString(Pixelation p) {
+    switch (p) {
+    case Pixelation::kHalfBlock: return "half block"; break;
+    case Pixelation::kQuarterBlock: return "quarter block"; break;
+    case Pixelation::kKittyGraphics: return "kitty graphics"; break;
+    case Pixelation::kiTerm2Graphics: return "iterm2 graphics"; break;
+#ifdef WITH_TIMG_SIXEL
+    case Pixelation::kSixelGraphics: return "sixel graphics"; break;
+#endif
+    case Pixelation::kNotChosen: return "(none)"; break;
+    }
+    return "";  // Make compiler happy.
+}
+
+// Print our version and various version numbers from our dependencies.
+static int PrintVersion(FILE *stream) {
+    fprintf(stream, "timg " TIMG_VERSION
+                    " <https://timg.sh/>\n"
+                    "Copyright (c) 2016..2024 Henner Zeller. "
+                    "This program is free software; license GPL 2.0.\n\n");
+#ifdef WITH_TIMG_GRPAPHICSMAGICK
+    fprintf(stream, "Image decoding %s\n",
+            timg::GraphicsMagickSource::VersionInfo());
+#endif
+#ifdef WITH_TIMG_OPENSLIDE_SUPPORT
+    fprintf(stream, "Openslide %s\n", timg::OpenSlideSource::VersionInfo());
+#endif
+#ifdef WITH_TIMG_JPEG
+    fprintf(stream, "Turbo JPEG\n");
+#endif
+#ifdef WITH_TIMG_RSVG
+    fprintf(stream, "librsvg %d.%d.%d + cairo %d.%d.%d\n",
+            LIBRSVG_MAJOR_VERSION, LIBRSVG_MINOR_VERSION, LIBRSVG_MICRO_VERSION,
+            CAIRO_VERSION_MAJOR, CAIRO_VERSION_MINOR, CAIRO_VERSION_MICRO);
+#endif
+#ifdef WITH_TIMG_POPPLER
+    fprintf(stream, "PDF rendering with poppler %s + cairo %d.%d.%d",
+            poppler_get_version(), CAIRO_VERSION_MAJOR, CAIRO_VERSION_MINOR,
+            CAIRO_VERSION_MICRO);
+#    if not POPPLER_CHECK_VERSION(0, 88, 0)
+    // Too old versions of poppler don't have a bounding-box function
+    fprintf(stream, " (no --auto-crop)");
+#    endif
+    fprintf(stream, "\n");
+#endif
+#ifdef WITH_TIMG_QOI
+    fprintf(stream, "QOI image loading\n");
+#endif
+#ifdef WITH_TIMG_STB
+    fprintf(stream,
+            "STB image loading; STB resize v"
+#    ifdef STB_RESIZE_VERSION2
+            "2"
+#    else
+            "1"
+#    endif
+#    ifdef WITH_TIMG_GRPAPHICSMAGICK
+            // If we have graphics magic, that will take images first,
+            // so STB will only really be called as fallback.
+            " (fallback)"
+#    endif
+            "\n");
+#endif
+    fprintf(stream, "swscale %s\n", AV_STRINGIFY(LIBSWSCALE_VERSION));
+#ifdef WITH_TIMG_VIDEO
+    fprintf(stream, "Video decoding %s\n", timg::VideoSource::VersionInfo());
+#endif
+#ifdef WITH_TIMG_SIXEL
+    fprintf(stream, "Libsixel version %s\n", LIBSIXEL_VERSION);
+#endif
+    fprintf(stream,
+            "Half, quarter, iterm2, and kitty graphics output: "
+            "timg builtin.\n");
+    return 0;
 }
 
 int main(int argc, char *argv[]) {
@@ -402,7 +503,7 @@ int main(int argc, char *argv[]) {
     present.terminal_use_upper_block =
         timg::GetBoolenEnv("TIMG_USE_UPPER_BLOCK");
 
-    const char *bg_color         = "auto";
+    std::string bg_color         = "auto";
     const char *bg_pattern_color = nullptr;
     display_opts.allow_frame_skipping =
         timg::GetBoolenEnv("TIMG_ALLOW_FRAME_SKIP");
@@ -418,6 +519,11 @@ int main(int argc, char *argv[]) {
     int geometry_height       = (term.rows - 2);
     bool debug_no_frame_delay = false;
 
+    if (auto pixelation_from_env = ParsePixelation(getenv("TIMG_PIXELATION"));
+        pixelation_from_env.has_value()) {
+        present.pixelation = *pixelation_from_env;
+    }
+
     // Convenience predicates: pixelation sending high-res images, no blocks.
     const auto is_pixel_direct_with_alpha = [](Pixelation p) {
         return p == Pixelation::kKittyGraphics ||
@@ -538,7 +644,7 @@ int main(int argc, char *argv[]) {
         case OPT_FRAME_OFFSET: frame_offset = atoi(optarg); break;
         case OPT_FRAME_COUNT: max_frames = atoi(optarg); break;
         case 'a': display_opts.antialias = false; break;
-        case 'b': bg_color = strdup(optarg); break;
+        case 'b': bg_color = std::string(optarg); break;
         case 'B': bg_pattern_color = strdup(optarg); break;
         case OPT_PATTERN_SIZE: display_opts.pattern_size = atoi(optarg); break;
         case OPT_SCROLL:
@@ -600,8 +706,15 @@ int main(int argc, char *argv[]) {
             break;
         case 'C': display_opts.center_horizontally = true; break;
         case 'U':
-            display_opts.upscale         = !display_opts.upscale;
-            display_opts.upscale_integer = (optarg && optarg[0] == 'i');
+            display_opts.upscale = !display_opts.upscale;
+            if (optarg) {
+                if (optarg[0] == 'i' || optarg[0] == 'I') {
+                    display_opts.upscale_integer = true;
+                }
+                else {
+                    fprintf(stderr, "Invalid parameter to --upscale\n");
+                }
+            }
             break;
         case OPT_AUTO_CROP:
             display_opts.auto_crop = true;
@@ -617,48 +730,7 @@ int main(int argc, char *argv[]) {
             break;
         case 'E': present.hide_cursor = false; break;
         case 'W': display_opts.fill_width = true; break;
-        case OPT_VERSION:
-            fprintf(stderr,
-                    "timg " TIMG_VERSION
-                    " <https://timg.sh/>\n"
-                    "Copyright (c) 2016..2023 Henner Zeller. "
-                    "This program is free software; license GPL 2.0.\n\n");
-#ifdef WITH_TIMG_GRPAPHICSMAGICK
-            fprintf(stderr, "Image decoding %s\n",
-                    timg::ImageLoader::VersionInfo());
-#endif
-#ifdef WITH_TIMG_OPENSLIDE_SUPPORT
-            fprintf(stderr, "Openslide %s\n",
-                    timg::OpenSlideSource::VersionInfo());
-#endif
-#ifdef WITH_TIMG_JPEG
-            fprintf(stderr, "Turbo JPEG\n");
-#endif
-#ifdef WITH_TIMG_QOI
-            fprintf(stderr, "QOI image loading\n");
-#endif
-#ifdef WITH_TIMG_STB
-            fprintf(stderr,
-                    "STB image loading"
-#    ifdef WITH_TIMG_GRPAPHICSMAGICK
-                    // If we have graphics magic, that will take images first,
-                    // so STB will only really be called as fallback.
-                    " fallback"
-#    endif
-                    "\n");
-#endif
-            fprintf(stderr, "swscale %s\n", AV_STRINGIFY(LIBSWSCALE_VERSION));
-#ifdef WITH_TIMG_VIDEO
-            fprintf(stderr, "Video decoding %s\n",
-                    timg::VideoLoader::VersionInfo());
-#endif
-            fprintf(stderr,
-                    "Half, quarter, iterm2, and kitty graphics output: "
-                    "timg builtin.\n");
-#ifdef WITH_TIMG_SIXEL
-            fprintf(stderr, "Libsixel version %s\n", LIBSIXEL_VERSION);
-#endif
-            return 0;
+        case OPT_VERSION: return PrintVersion(stdout);
         case OPT_TITLE:
             display_opts.show_title = !display_opts.show_title;
             if (optarg) display_opts.title_format = optarg;
@@ -678,15 +750,10 @@ int main(int argc, char *argv[]) {
             }
             break;
         case 'p':
-            switch (tolower(optarg[0])) {  // NOLINT
-            case 'h': present.pixelation = Pixelation::kHalfBlock; break;
-            case 'q': present.pixelation = Pixelation::kQuarterBlock; break;
-            case 'k': present.pixelation = Pixelation::kKittyGraphics; break;
-            case 'i': present.pixelation = Pixelation::kiTerm2Graphics; break;
-#ifdef WITH_TIMG_SIXEL
-            case 's': present.pixelation = Pixelation::kSixelGraphics; break;
-#endif
-            default:
+            if (const auto p = ParsePixelation(optarg); p.has_value()) {
+                present.pixelation = *p;
+            }
+            else {
                 fprintf(stderr, "Unknown --pixelation/-p parameter '%s'\n",
                         optarg);
             }
@@ -763,7 +830,6 @@ int main(int argc, char *argv[]) {
     // Determine best default to pixelate images.
     if (present.pixelation == Pixelation::kNotChosen) {
         present.pixelation = Pixelation::kQuarterBlock;  // Good default.
-#ifdef WITH_TIMG_TERMINAL_QUERY
         if (term.font_width_px > 0 && term.font_height_px > 0) {
             auto graphics_info      = timg::QuerySupportedGraphicsProtocol();
             present.tmux_workaround = graphics_info.in_tmux;
@@ -775,25 +841,24 @@ int main(int argc, char *argv[]) {
                 present.pixelation = Pixelation::kKittyGraphics;
                 break;
             case timg::GraphicsProtocol::kSixel:
-#    ifdef WITH_TIMG_SIXEL
+#ifdef WITH_TIMG_SIXEL
                 present.pixelation = Pixelation::kSixelGraphics;
                 present.sixel_cursor_workaround =
                     graphics_info.known_broken_sixel_cursor_placement;
-#    else
+#else
                 present.pixelation = Pixelation::kQuarterBlock;
-#    endif
+#endif
                 break;
             case timg::GraphicsProtocol::kNone: break;
             }
         }
-#endif
     }
     else if (present.pixelation == Pixelation::kKittyGraphics) {
         // If the user manually chooses kitty, we still need to know if in tmux
         auto graphics_info      = timg::QuerySupportedGraphicsProtocol();
         present.tmux_workaround = graphics_info.in_tmux;
     }
-#if defined(WITH_TIMG_SIXEL) && defined(WITH_TIMG_TERMINAL_QUERY)
+#if defined(WITH_TIMG_SIXEL)
     // If the user manually choose sixel, we still can't avoid a terminal
     // query, as we have to figure out if it has a broken cursor implementation.
     else if (present.pixelation == Pixelation::kSixelGraphics) {
@@ -806,14 +871,14 @@ int main(int argc, char *argv[]) {
     // The high-res image terminals provide alpha-blending, no need to
     // query the terminal color for 'auto'
     if (is_pixel_direct_with_alpha(present.pixelation) &&
-        (strcasecmp(bg_color, "auto") == 0)) {
+        (strcasecmp(bg_color.c_str(), "auto") == 0)) {
         bg_color = "none";
     }
 
     // If 'none' is chosen for background color, then emitting the
     // PNG with alpha channels gives us compositing on client side.
     if (is_pixel_direct_p(present.pixelation) &&
-        (strcasecmp(bg_color, "none") == 0)) {
+        (strcasecmp(bg_color.c_str(), "none") == 0)) {
         display_opts.local_alpha_handling = false;
     }
 
@@ -904,30 +969,30 @@ int main(int argc, char *argv[]) {
 
     // Asynconrous image loading (filelist.size()) and terminal query (+1)
     thread_count = (thread_count > 0 ? thread_count : kDefaultThreadCount);
+
+    // Note: this thread pool will be leaked explicitly to not unnecessarily
+    // have to wait on potentially blocking cleanup at program exit where it
+    // does not matter.
     timg::ThreadPool *const pool =
         new timg::ThreadPool(std::min(thread_count, (int)filelist.size() + 1));
 
     std::future<rgba_t> background_color_future;
-    if (bg_color) {
-        if (strcasecmp(bg_color, "auto") == 0) {
-#ifdef WITH_TIMG_TERMINAL_QUERY
-            std::function<rgba_t()> query_terminal = []() {
-                return rgba_t::ParseColor(timg::QueryBackgroundColor());
-            };
-            background_color_future     = pool->ExecAsync(query_terminal);
-            display_opts.bgcolor_getter = [&background_color_future]() {
-                static rgba_t value = background_color_future.get();  // once
-                return value;
-            };
-#else
-            const rgba_t bg             = rgba_t::ParseColor("#000000");
-            display_opts.bgcolor_getter = [bg]() { return bg; };
-#endif
-        }
-        else {
-            const rgba_t bg             = rgba_t::ParseColor(bg_color);
-            display_opts.bgcolor_getter = [bg]() { return bg; };
-        }
+    if (strcasecmp(bg_color.c_str(), "auto") == 0) {
+        std::function<rgba_t()> query_terminal = []() {
+            return rgba_t::ParseColor(timg::QueryBackgroundColor());
+        };
+        // Finding the background color might take a while, so we query
+        // it asynchonously and only force a wait on it once an image display
+        // actually queries it.
+        background_color_future     = pool->ExecAsync(query_terminal);
+        display_opts.bgcolor_getter = [&background_color_future]() {
+            static rgba_t value = background_color_future.get();  // once
+            return value;
+        };
+    }
+    else {
+        const rgba_t bg             = rgba_t::ParseColor(bg_color.c_str());
+        display_opts.bgcolor_getter = [bg]() { return bg; };
     }
 
     display_opts.bg_pattern_color = rgba_t::ParseColor(bg_pattern_color);
@@ -977,8 +1042,9 @@ int main(int argc, char *argv[]) {
         output_fd, buffer_allow_skipping, kAsyncWriteQueueSize,
         debug_no_frame_delay, interrupt_received);
     const Time start_show = Time::Now();
-    PresentImages(&loaded_sources, display_opts, present, &sequencer,
-                  &cell_size_warning_needed);
+    const int successful_images =
+        PresentImages(&loaded_sources, display_opts, present, &sequencer,
+                      &cell_size_warning_needed);
     const Time end_show = Time::Now();
 
     // Error messages have been collected to not clutter the image output
@@ -1007,7 +1073,7 @@ int main(int argc, char *argv[]) {
         if (term.font_width_px < 0 || term.font_height_px < 0) {
             fprintf(stderr,
                     "Note: Terminal does not return ws_xpixel and ws_ypixel "
-                    "in TIOCGWINSZ ioctl.\n"
+                    "in TIOCGWINSZ ioctl or \"\\033[16t\" query.\n"
                     "\t->Aspect ratio might be off.\n"
                     "\t->File a feature request with the terminal "
                     "emulator program you use\n");
@@ -1022,18 +1088,8 @@ int main(int argc, char *argv[]) {
         }
         fprintf(stderr, "\n");
 
-        const char *pixelation = "";
-        switch (present.pixelation) {
-        case Pixelation::kHalfBlock: pixelation = "half block"; break;
-        case Pixelation::kQuarterBlock: pixelation = "quarter block"; break;
-        case Pixelation::kKittyGraphics: pixelation = "kitty graphics"; break;
-        case Pixelation::kiTerm2Graphics: pixelation = "iterm2 graphics"; break;
-#ifdef WITH_TIMG_SIXEL
-        case Pixelation::kSixelGraphics: pixelation = "sixel graphics"; break;
-#endif
-        case Pixelation::kNotChosen: pixelation = "(none)"; break;
-        }
-        fprintf(stderr, "Using %s pixels", pixelation);
+        fprintf(stderr, "Effective pixelation: Using %s",
+                PixelationToString(present.pixelation));
 #ifdef WITH_TIMG_SIXEL
         if (present.pixelation == Pixelation::kSixelGraphics) {
             if (present.sixel_cursor_workaround) {
@@ -1044,10 +1100,16 @@ int main(int argc, char *argv[]) {
             }
         }
 #endif
+        if (present.pixelation == Pixelation::kKittyGraphics) {
+            if (present.tmux_workaround) {
+                fprintf(stderr, " (with tmux workaround)");
+            }
+        }
         fprintf(stderr, ".\n");
         const rgba_t bg = display_opts.bgcolor_getter();
-        fprintf(stderr, "Background color for transparency '%s'", bg_color);
-        if (strcasecmp(bg_color, "none") != 0) {
+        fprintf(stderr, "Background color for transparency '%s'",
+                bg_color.c_str());
+        if (strcasecmp(bg_color.c_str(), "none") != 0) {
             fprintf(stderr, "; effective RGB #%02x%02x%02x", bg.r, bg.g, bg.b);
         }
         fprintf(stderr, "\n");
@@ -1062,9 +1124,10 @@ int main(int argc, char *argv[]) {
         const uint64_t written_bytes =
             sequencer.bytes_total() - sequencer.bytes_skipped();
         fprintf(stderr,
-                "%d file%s; %s written (%s/s) "
+                "%d file%s (%d successful); %s written (%s/s) "
                 "%" PRId64 " frames",
                 (int)filelist.size(), filelist.size() == 1 ? "" : "s",
+                successful_images,
                 timg::HumanReadableByteValue(written_bytes).c_str(),
                 timg::HumanReadableByteValue(written_bytes / d).c_str(),
                 sequencer.frames_total());
@@ -1079,15 +1142,29 @@ int main(int argc, char *argv[]) {
                 100.0 * sequencer.frames_skipped() / sequencer.frames_total());
         }
         fprintf(stderr, "\n");
+
+        auto print_env = [](const char *env) {
+            const char *value = getenv(env);
+            fprintf(stderr, " %-23s%s", env, value ? " = " : "   (not set)\n");
+            if (value) fprintf(stderr, "\"%s\"\n", value);
+        };
+        fprintf(stderr, "Environment variables\n");
+        print_env("TIMG_PIXELATION");
+        print_env("TIMG_DEFAULT_TITLE");
+        print_env("TIMG_ALLOW_FRAME_SKIP");
+        print_env("TIMG_USE_UPPER_BLOCK");
+        print_env("TIMG_FONT_WIDTH_CORRECT");
     }
 
     if (cell_size_unknown_in_pixel_mode && cell_size_warning_needed) {
         fprintf(stderr,
                 "Terminal does not support pixel size query, "
-                "but graphics protocol requested that needs that info.\n"
+                "but with %s this is needed to show animations or columns.\n"
                 "File an issue with your terminal implementation to implement "
-                "ws_xpixel, ws_ypixel on TIOCGWINSZ\n"
-                "Can't show animations or have columns in grid.\n");
+                "ws_xpixel, ws_ypixel on TIOCGWINSZ or \"\\033[16t\" query.\n"
+                "Can't show animations or have columns in grid.\n(Suggestion: "
+                "switch back to --pixelation=quarter for now)\n",
+                PixelationToString(present.pixelation));
     }
 
     // If we were super-fast decoding and showing images that didn't need
@@ -1097,6 +1174,8 @@ int main(int argc, char *argv[]) {
     if (background_color_future.valid())
         background_color_future.wait_for(std::chrono::milliseconds(200));
 
+    free((void *)bg_pattern_color);
+
     // Deliberately leaking thread pool as we don't bother waiting for
     // lingering threads
 
