@@ -317,12 +317,12 @@ char *UnicodeBlockCanvas::AppendDoubleRow(char *pos, int indent, int width,
 
 void UnicodeBlockCanvas::Send(int x, int dy, const Framebuffer &framebuffer,
                               SeqType seq_type, Duration end_of_frame) {
-    const int width          = framebuffer.width();
-    const int height         = framebuffer.height();
-    char *const start_buffer = RequestBuffers(width, height);
-    char *pos                = start_buffer;
+    const int width  = framebuffer.width();
+    const int height = framebuffer.height();
+    OutBuffer out_buffer(RequestBuffers(width, height), 0);
+    char *pos = out_buffer.data;
 
-    if (dy < 0) MoveCursorDY((dy - 1) / 2);
+    if (dy < 0) MoveCursorDY(cell_height_for_pixels(dy));
 
     pos = AppendPrefixToBuffer(pos);
 
@@ -383,15 +383,17 @@ void UnicodeBlockCanvas::Send(int x, int dy, const Framebuffer &framebuffer,
     last_framebuffer_height_ = height;
     last_x_indent_           = x;
     if (before_image_emission == pos) {
-        // Don't even emit cursor up/dn jump, but make sure to return buffer.
-        write_sequencer_->WriteBuffer(start_buffer, 0, seq_type, end_of_frame);
+        // Don't even emit cursor up/dn jump, keep buffer size zero.
+        write_sequencer_->WriteBuffer(std::move(out_buffer), seq_type,
+                                      end_of_frame);
         return;
     }
 
     if (y_skip) {
         pos += sprintf(pos, SCREEN_CURSOR_DN_FORMAT, y_skip);
     }
-    write_sequencer_->WriteBuffer(start_buffer, pos - start_buffer, seq_type,
+    out_buffer.size = (size_t)(pos - out_buffer.data);
+    write_sequencer_->WriteBuffer(std::move(out_buffer), seq_type,
                                   end_of_frame);
 }
 
@@ -417,19 +419,20 @@ char *UnicodeBlockCanvas::RequestBuffers(int width, int height) {
                                + SCREEN_END_OF_LINE_LEN);  // Finishing a line.
 
     // Depending on even/odd situation, we might need one extra row.
-    const size_t new_backing = width * (height + 1) * sizeof(rgba_t);
+    // For quarter, we have one extra possible pixel wider.
+    const size_t new_backing = (width + 1) * (height + 1) * sizeof(rgba_t);
     if (new_backing > backing_buffer_size_) {
         backing_buffer_      = (rgba_t *)realloc(backing_buffer_, new_backing);
         backing_buffer_size_ = new_backing;
     }
 
-    const size_t new_empty = width * sizeof(rgba_t);
+    const size_t new_empty = (width + 1) * sizeof(rgba_t);
     if (new_empty > empty_line_size_) {
         empty_line_      = (rgba_t *)realloc(empty_line_, new_empty);
         empty_line_size_ = new_empty;
         memset(empty_line_, 0x00, empty_line_size_);
     }
-    return write_sequencer_->RequestBuffer(content_size);
+    return new char[content_size];
 }
 
 // Converting the colors requires fast uint8 -> ASCII decimal digits with
